// Copyright (C) 2000 Dominic Letourneau (doumdi@yahoo.com)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this file.  If not, write to the Free Software Foundation,
// 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "ChannelSeparator.h"
#include "GreyScale8Image.h"

NODE_INFO(ChannelSeparator,"Image", "IMAGE_IN", "RED_OUT:GREEN_OUT:BLUE_OUT", "")

ChannelSeparator::ChannelSeparator(string nodeName, ParameterSet params)
: Node(nodeName, params)
{
  // Add the node's input
  m_IMAGE_ID = addInput("IMAGE_IN");
  
  // Add the node's output
  addOutput("RED_OUT");
  addOutput("GREEN_OUT");
  addOutput("BLUE_OUT");
  
}

ChannelSeparator::~ChannelSeparator()
{
  return;
}
ObjectRef ChannelSeparator::getOutput (int output_id, int count) {

  if (!hasOutput(output_id)) throw new NodeException (this, "Cannot getOutput id",__FILE__,__LINE__);
  
   
  if (count != processCount) {
    //We are updating our output only if needed
    
    try {
      
      //getting all data from our inputs.
      int OutputID = inputs[m_IMAGE_ID].outputID;
      ObjectRef im = inputs[m_IMAGE_ID].node->getOutput(OutputID, count);


      RGB24Image *im_ptr =  dynamic_cast<RGB24Image *>(im.get());
      if (!im_ptr) {
	throw new NodeException (this, "Cannot getOutput source image",__FILE__,__LINE__);
      }
      
      unsigned char *src_ptr = im_ptr->get_data();

      GreyScale8Image *red = new GreyScale8Image(im_ptr->get_width(), im_ptr->get_height());
      GreyScale8Image *green = new GreyScale8Image(im_ptr->get_width(), im_ptr->get_height());
      GreyScale8Image *blue = new GreyScale8Image(im_ptr->get_width(), im_ptr->get_height());
      
      unsigned char *red_data = red->get_data();
      unsigned char *green_data = green->get_data();
      unsigned char *blue_data = blue->get_data();
     
      for (int i =0; i <im_ptr->get_height(); i++) {
	for (int j = 0; j <im_ptr->get_width(); j++) {
	  *red_data = *src_ptr++;
	  red_data++;
	  *green_data = *src_ptr++;
	  green_data++;
	  *blue_data = *src_ptr++;
	  blue_data++;
	  src_ptr++;
	}
      }
      
      
      m_RED = ObjectRef(red);
      m_GREEN = ObjectRef(green);
      m_BLUE = ObjectRef(blue);
      
    } //end of try block
    catch (BaseException *e) {
      //Something weird happened
      //e->print();
      throw e->add(new NodeException (this,
				      string("error caught in ChannelSeparator::getOutput(int,int)"), 
				      __FILE__,__LINE__));
     }      
     
    //updating processCount
    processCount = count;                 
  }

  
    switch (output_id){
      
    case 0:
      return m_RED;
      break;
    case 1:
      return m_GREEN;
      break;
    case 2:
      return m_BLUE;
      break;
      
    } 
  }
