#ifndef _GREYSCALE8IMAGE_CC_
#define _GREYSCALE8IMAGE_CC_

#include "GreyScale8Image.h"
#include <string>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include "errno.h"

//unsigned char *m_data_buffer;
//int m_width;
//int m_height;
bool GreyScale8Image::write_ppm (string &filename) {
    
  unsigned char *p;
  char header[256];
  int fd;


  sprintf (header, "P5\n%d %d\n255\n", m_width, m_height);
  
  if ((fd = open (filename.c_str(), O_WRONLY | O_CREAT, 0644)) < 0) {
    printf ("ppm open failed (writing): %d\n", errno);
    return false;
  }
  
  write (fd, &header[0], strlen(header));

  p = &m_data[0];
  for (int j=0; j<m_height; j++) {
    for (int k=0; k<m_width; k++) {	
      write(fd,p++, 1);	
    }
  }

  close (fd);
  return true;
}

bool GreyScale8Image::read_ppm (string &filename) {
  return true;
}
void GreyScale8Image::draw_rect(CRect &rect, unsigned char val) {

  CPoint ulc = rect.TopLeft();
  CPoint lrc = rect.BottomRight();

  //boundary check
  if (ulc.x < 0 ||
      ulc.y < 0 ||
      ulc.x >= m_width ||
      ulc.y >= m_height) {
    char message[256];
    sprintf(message,"CRect (ulc) mismatch x:%i y:%x m_width:%i m_height:%i",lrc.x,lrc.y,m_width,m_height);
    throw new GeneralException(message,__FILE__,__LINE__);
  }

  //boundary check
  if (lrc.x < 0 ||
      lrc.y < 0 ||
      lrc.x >= m_width ||
      lrc.y >= m_height) {

    char message[256];
    sprintf(message,"CRect (lrc) mismatch x:%i y:%x m_width:%i m_height:%i",lrc.x,lrc.y,m_width,m_height);
    throw new GeneralException(message,__FILE__,__LINE__);
  }

  unsigned char *p = NULL;
  //drawing vertical
  for (int i = ulc.y; i <= lrc.y;i++) {
    set_val(ulc.x,i,val);
    set_val(lrc.x,i,val);
  }
  //drawing horizontal
  for (int j = ulc.x; j <= lrc.x; j++) {
    set_val(j,ulc.y,val);
    set_val(j,lrc.y,val);    
  }



}

void GreyScale8Image::create(int width, int height){

  if (m_data) {
    //should throw an exception
    //or resize ?
  }
  else {
    m_data = new unsigned char [width * height];
    m_width = width;
    m_height = height;
  }

}

void GreyScale8Image::destroy() {
  if (m_data) {
    delete [] m_data;
    m_data = NULL;
    m_width = 0;
    m_height = 0;
  }
}

GreyScale8Image::GreyScale8Image()
  : Image(GREYSCALE) 
{
  m_data = NULL;
  m_width = 0;
  m_height = 0;

}

GreyScale8Image::GreyScale8Image(int width, int height) 
  :Image(GREYSCALE)
{
  create(width, height);
}

GreyScale8Image::GreyScale8Image(int width, int height, const unsigned char val) 
  :Image(GREYSCALE)
{
  create(width,height);
  initialize(val);
}

GreyScale8Image::GreyScale8Image(const GreyScale8Image &image) 
  : Image(GREYSCALE) {
  create(image.m_width, image.m_height);
  memcpy(m_data, image.m_data, m_width * m_height);
}

GreyScale8Image::GreyScale8Image(int width, int height, unsigned char *buffer)
  : Image(GREYSCALE) 
{
  create(width, height);
  memcpy(m_data, buffer, m_width * m_height);
}

void GreyScale8Image::initialize (const unsigned char val) {
  if (m_data) {
    memset(m_data,val,m_width * m_height);
  }

}

GreyScale8Image::~GreyScale8Image() {
  destroy();
}

void GreyScale8Image::resize (int width, int height) {

  unsigned char *tmp = m_data;
  int old_width = m_width;
  int old_height =m_height;
  unsigned char val;

  create(width,height);

  //copying the old image
  if (tmp) {

    for (int i = 0; i < m_height && i < old_height; i++) {
      for (int j = 0; j < m_width && j < old_width; j++) {
	val = get_byte (i,j, tmp,old_width, old_height);
	set_byte(i,j,val,m_data, m_width, m_height);	
      }
    }
    
    delete [] tmp;
    
  }
}



#endif
